!
! Sim_Forward
!
! Program to provide an example of CRTM Forward function usage.
!
!
! CREATION HISTORY:
!       Example Written by:     Paul van Delst, 08-Jan-2009
!                       paul.vandelst@noaa.gov
!       Sample program by :   David Groff
!       Written for DBL91 to simulate simulated obs
!                        Michiko Masutani  Nov-2009
!                        Supported by David Groff 
!       
!

PROGRAM Sim_Forward

  ! ============================================================================
  ! **** ENVIRONMENT SETUP FOR RTM USAGE ****
  !
  ! Module usage
  USE CRTM_Module
  USE SpcCoeff_Define
  ! Disable all implicit typing
  IMPLICIT NONE
  ! ============================================================================


  ! ----------
  ! Parameters
  ! ----------
  CHARACTER(*), PARAMETER :: PROGRAM_NAME   = 'Sim_Forward'
  CHARACTER(*), PARAMETER :: PROGRAM_RCS_ID = &
    '$Id: Example_Forward.f90 2984 2009-01-08 18:01:55Z paul.vandelst@noaa.gov $'



  ! ============================================================================
  ! 0. **** SOME SET UP PARAMETERS FOR THIS EXAMPLE ****
  !
  INTEGER, PARAMETER :: Max_Profiles  = 100  !  number of profiles processed at one time
  INTEGER, PARAMETER :: Tot_Profiles  = 100000 ! limit of profiles to be processed
  INTEGER, PARAMETER :: Max_Layers    = 91
  INTEGER, PARAMETER :: Max_ABSORBERS = 2
  INTEGER, PARAMETER :: Max_CLOUDS    = 2
  INTEGER, PARAMETER :: Max_AEROSOLS  = 2
  ! ...but only ONE Sensor at a time
  INTEGER, PARAMETER :: Max_Sensors = 2    ! could be more than 1 for AQUA
  INTEGER, PARAMETER :: Max_Channels =  1000 

  ! Test GeometryInfo angles. The test scan angle is based
  ! on the default Re (earth radius) and h (satellite height)                                                      
!  REAL(fp_kind), PARAMETER :: ZENITH_ANGLE 
!  REAL(fp_kind), PARAMETER :: SCAN_ANGLE  
  ! ============================================================================
  
  
  ! ---------
  ! Variables
  ! ---------
  CHARACTER(30),DIMENSION(Max_Sensors) :: Sensor_ID
  INTEGER,DIMENSION(Max_Sensors) :: luprof,lurad
  CHARACTER(256) :: Message_Log
  INTEGER :: Error_Status1,Error_Status2,Error_Status3,Error_Status
  INTEGER :: Allocate_Status
  INTEGER :: totn_Channels
  INTEGER :: N_Sensors 
  INTEGER :: Total_Profiles
  INTEGER :: k1, k2, l, m,NC1,NC2,i,j,k
  INTEGER :: Sensor, Channel
  INTEGER :: N_Profiles    ! number of profiles read
  INTEGER :: init,IPRT
  INTEGER :: iproc, iprof_write

  ! ============================================================================
  ! 1. **** DEFINE THE CRTM INTERFACE STRUCTURES ****
  !
  TYPE(CRTM_ChannelInfo_type) , DIMENSION(Max_Sensors)        :: ChannelInfo
  TYPE(CRTM_GeometryInfo_type), DIMENSION(Max_Profiles)       :: GeometryInfo

  ! Define the FORWARD variables
  TYPE(CRTM_Atmosphere_type)  , DIMENSION(Max_Profiles)       :: Atm
  TYPE(CRTM_Surface_type)     , DIMENSION(Max_Profiles)       :: Sfc
  TYPE(CRTM_Options_type)     ,DIMENSION(Max_Profiles)        :: Options
  !TYPE(SpcCoeff_type),DIMENSION(Max_Sensors)::SpcCoeff
  CHARACTER(80):: RCS_Id
  TYPE(CRTM_RTSolution_type)  , DIMENSION(:,:), ALLOCATABLE :: RTSolution
  ! ============================================================================
  !  Variables added by  MSQ
  INTEGER :: EOF1
  REAL(fp_kind):: total_aerosol,total_clw
  REAL(fp_kind),DIMENSION(Max_Profiles)::hour
  INTEGER,DIMENSION(Max_Profiles):: year,month,day
  REAL(fp_kind):: atmp
  INTEGER,DIMENSION(Max_Profiles)::crtm_stype

  ! data lusfcvar/10/  used   read_DBL91
  INTEGER::luwk1,luwk2
  INTEGER::sat_type   !  sat_type will be used to control surface etc
!   
!  sat type 1: Micro 2; IR Hirs 3: GOES  4:AIRS IASI
!

  data luwk1/60/luwk2/61/    ! not passed also defined in each program
  data luprof/20,21/ ! DBL91
  data lurad/30,31/  ! ascii out put
  data IPRT/5/


  ! Program header
  ! --------------
  CALL Program_Message( PROGRAM_NAME, &
                        'Simulate Radiance form DBL91', &
                        '$Revision: 2984 $' )
  ! Get sensor id from user
  ! -----------------------
  print*,'  start simulation'
  open(luwk1,form='formatted',status='unknown')
  open(luwk2,form='formatted',status='unknown')
  read *,iprt
  read *,sat_type
  print*,' iprt sat_type',iprt,sat_type
  do sensor=1,Max_Sensors
  READ( *,'(a30)',IOSTAT=Error_Status ) Sensor_ID(sensor)
  open(lurad(sensor),form='formatted',status='unknown')
  if (Error_Status/=0) then
      m=sensor-1
      exit
   endif
   Sensor_ID(sensor) = ADJUSTL(Sensor_ID(sensor))
  print*,' Sensor_ID ',sensor,Sensor_ID(sensor)
  enddo
  N_Sensors=m
  print*,'Number of Sensor  ',N_Sensors

  Atm%Max_Layers=Max_Layers
  Atm%Max_Clouds=Max_Clouds
  Atm%Max_Aerosols=Max_Aerosols

  ! ============================================================================
  ! 2. **** INITIALIZE THE CRTM ****
  !
  ! 2a. This initializes the CRTM for the sensors 
  !     predefined in the example SENSOR_ID parameter.
  !     NOTE: The coefficient data file path is hard-
  !           wired for this example.
  ! --------------------------------------------------
  WRITE( *,'(/5x,"Initializing the CRTM...")' )
  Error_Status = CRTM_Init( ChannelInfo(1:N_Sensors)   , &  ! This is an OUTPUT
                            Sensor_ID(1:N_Sensors), &
                            File_Path='../Coefficient_Data/')
  IF ( Error_Status /= SUCCESS ) THEN 
    CALL Display_Message( PROGRAM_NAME, &
                          'Error initializing CRTM', & 
                          Error_Status)  
    STOP
  END IF
  Print*,' CRTM initialization SUCCESS'

  ! 2b. Determine the total number of channels
  !     for which the CRTM was initialized
  ! ------------------------------------------
  totn_Channels = SUM(ChannelInfo%n_Channels)
  print*,' totn_Channels =',totn_Channels
  !
  !
  ! ============================================================================
  ! 3. **** ALLOCATE STRUCTURE ARRAYS ****
  !
  ALLOCATE( RTSolution( totn_Channels, Max_PROFILES ), STAT=Allocate_Status )
  IF ( Allocate_Status /= 0 ) THEN
    CALL Display_Message( PROGRAM_NAME, &
                          'Error allocating structure arrays', &
                           Error_Status)
    STOP
  END IF


  ! 3b. Allocate the STRUCTURES
  ! ---------------------------
  ! The input FORWARD structure
  Error_Status1 = CRTM_Allocate_Atmosphere( Max_LAYERS, Max_ABSORBERS, &
                             Max_CLOUDS, Max_AEROSOLS, &
                                           Atm )
  Error_Status2 = CRTM_Allocate_RTSolution( Max_LAYERS, RTSolution)
  Error_Status3 = CRTM_Allocate_Options( Max_Channels, Options   )
  ERROR_Status=ERROR_Status1+Error_Status2+Error_Status3
  IF ( Error_Status /= SUCCESS ) THEN 
     CALL Display_Message( PROGRAM_NAME, &
                           'Error allocating CRTM Atmosphere structures', & 
                           Error_Status)  
     print*,' Error_Status ',ERROR_Status1,Error_Status2,Error_Status3
   STOP
  END IF
  ! ============================================================================
  ! ============================================================================
  ! 4. **** ASSIGN INPUT DATA ****
  !
  ! Fill the Atm structure array. 
  ! NOTE: This is an example program for illustrative purposes only.
  !       Typically, one would not assign the data as shown below,
  !       but rather read it from file
  ! ============================================================================
  do sensor=1,N_Sensors
  print*,' start working on sensor ',sensor,Sensor_ID(sensor)
  EOF1=0
  init=1
  iprof_write=0
  Total_profiles=0 
  iproc=0 
  do   !  read  and process Max_Profiles at a time
  iproc=iproc+1   ! this could be used for multi tasking
  N_Profiles=0    ! profiles processed in this loop
  do m=1,Max_Profiles 
  ! ============================================================================
   Call  Read_NR_Record( luprof(sensor),init,Sensor_ID(sensor:sensor),& !input
                      sat_type, &  ! Input
                      Atm(m) , &  ! In and Output
                      Sfc(m) , &   ! Out put
                      GeometryInfo(m), &   ! Output
                      m,&    ! no used as an index but added for checking
                      Message_Log,&  ! Error messaging
                      year(m),month(m),day(m),hour(m),crtm_stype(m), &
                      EOF1,IPRT)!  
!
! $$$$$$$$$$$$$$$$$$$$$$$  no cloud experiments $$$$$$$$$$$$$$$$$$$$$$$$$$
!
!        Atm(m)%N_Clouds=0
!
! $$$$$$$$$$$$$$$$$$$$$$$  no cloud experiments $$$$$$$$$$$$$$$$$$$$$$$$$$
!

    if (EOF1==1) then
      print*,' EOF1= ',EOF1,'  m=',m
      N_Profiles=m-1
      print*,' EOF encountored  iproc=',iproc
      print*,' EOF encountored  N_profiles=',N_profiles
      print*,' EOF encountored  Total_profiles=',Total_profiles
      exit
    endif
      N_profiles=m
      Total_Profiles=Total_Profiles+1
   if (Total_profiles.GE.Tot_profiles) then
      print*,Total_profiles ,'=',Tot_Profiles,' read.  finish reading'
      print*,'  N_profiles=',N_profiles
      EOF1=1
      exit
   endif
  enddo 
  if (iprt.ge.0) then
  print*,'iproc =',iproc,&
          ' number of profile read for this iproc= ',&
          N_profiles,'Total read =',Total_profiles
  endif
  


  ! 5. **** CALL THE CRTM FORWARD MODEL ****
  !
  if (IPRT.ge.3) print*,' call CRTM_Forward'
  NC1=1
  do i=1,sensor-1
  NC1=NC1+ChannelInfo(i)%n_Channels
  enddo
  NC2=NC1+ChannelInfo(sensor)%n_Channels-1
  if (iprt.ge.3)Print*,' NC1,NC2, ChannelInfo(sensor)%n_Channels',&
          NC1,NC2, ChannelInfo(sensor)%n_Channels
  Error_Status = CRTM_Forward( Atm(1:N_Profiles)     , &  
                               Sfc(1:N_Profiles)         , &  
                               GeometryInfo(1:N_Profiles), &  
                               ChannelInfo(sensor:sensor), &  
                               RTSolution(NC1:NC2,1:N_Profiles))
  if (iprt.ge.3) print*,' return from CRTM   ',Error_Status
 


  IF ( Error_Status /= SUCCESS ) THEN 
    CALL Display_Message( PROGRAM_NAME, &
                          'Error in CRTM Forward Model', & 
                          Error_Status)  

    STOP
  END IF
  ! ============================================================================
  ! 6. **** OUTPUT THE RESULTS TO SCREEN ****
  !
  DO m = 1, N_Profiles
    iprof_write=iprof_write+1
    if (iprt.ge.3) then
    WRITE( *,1000)iprof_write
    1000 format(/7x,'Profile ',i10,' FWD output',&
         & /7x,'Sensor_ID',3x,'Channel',9x,'R',12x,'Tb',3i6)
     endif
     if (iprt.ge.4) then
     print*,' iprof_write =',iprof_write,m,N_Profiles,Total_Profiles,EOF1
     endif
     if (iprt.ge.3) then
     write(*,'(I10,A15,7I10)') iprof_write,Sensor_ID(sensor),&
                      crtm_stype(m),year(m),month(m),day(m),&
                      ChannelInfo(sensor)%n_Channels
     write(*,'(9f10.3)') GeometryInfo(m)%Latitude,&
                        GeometryInfo(m)%Longitude,&
                        hour(m),&
                        GeometryInfo(m)%Sensor_Zenith_Angle, &
                        GeometryInfo(m)%Source_Zenith_Angle
     endif
     if (iprt.ge.1) then
             write(luwk1,'(40x,i5,3f10.3)')&
                   iprof_write, GeometryInfo(m)%Longitude,&
                   GeometryInfo(m)%Latitude,&
                   hour(m)
     endif
     write(lurad(sensor),'(I10,A15,7I10)') iprof_write,Sensor_ID(sensor),&
                      crtm_stype(m),year(m),month(m),day(m),&
                      ChannelInfo(sensor)%n_Channels
         
     write(lurad(sensor),'(9f10.3)') &
              GeometryInfo(m)%Latitude,&
              GeometryInfo(m)%Longitude,&
              hour(m),&
              GeometryInfo(m)%Sensor_Zenith_Angle, &
              GeometryInfo(m)%Source_Zenith_Angle
      total_clw = 0.0
      total_aerosol = 0.0
      DO i = 1, Atm(m)%n_Layers
        do j=1,Atm(m)%N_Clouds
       total_clw =total_clw+Atm(m)%Cloud(j)%Water_Content(i)
        enddo
        do j=1,Atm(m)%N_Aerosols
          atmp = (Atm(m)%Level_Pressure(i)-Atm(m)%Level_Pressure(i-1))&
                  *100.0/9.8 * 1000.0  * 1.0
          total_aerosol = total_aerosol + atmp*Atm(m)%Aerosol(j)%Concentration(i)
        enddo
     ENDDO   ! i Layers
      if (iprt.ge.4) then
        print*,' total_clw,total_aerosol ',total_clw,total_aerosol
      endif
      if (iprt.ge.4)print*,'sensor,NC1,NC2',Sensor,NC1,NC2

    DO l = NC1,NC2
      ! Determine the ChannelInfo sensor 
      Channel=l-NC1+1
      if (iprt.ge.6) then
        print*,' channel=',channel,' l=',l
      endif
      IF ( Channel > totn_Channels ) THEN
         print*, 'Number of Channel ',Channel ,' exceed ',&
          totn_Channels 
         stop
      END IF
      if (IPRT.ge.4) then
      WRITE( *,'(7x,a9,5x,i2,5x,es13.6,5x,f7.3)') ChannelInfo(Sensor)%Sensor_ID, &
                       ChannelInfo(Sensor)%Sensor_Channel(Channel), &
                        RTSolution(l,m)%Radiance, &
                        RTSolution(l,m)%Brightness_Temperature
      endif
      !WRITE( lurad(sensor),'(7x,a9,5x,i2,5x,es13.6,5x,f7.3)') ChannelInfo(Sensor)%Sensor_ID, &
      !                 ChannelInfo(Sensor)%Sensor_Channel(Channel), &
      !                  RTSolution(l,m)%Radiance, &
      !                  RTSolution(l,m)%Brightness_Temperature
      WRITE( lurad(sensor),'(i6,f10.3)') &
                       ChannelInfo(Sensor)%Sensor_Channel(Channel), &
                        RTSolution(l,m)%Brightness_Temperature

     END DO   ! l



  END DO  ! m
  ! ============================================================================
  print*,Sensor_ID(sensor),'iproc, total profile written ',iproc,iprof_write
  if (EOF1 ==1) then
      print*,' EOF encountored',iproc,iprof_write
      exit
  endif

  END DO !   # Max_Profiles data read    #  This is to check EOF

  print*,' finish processing Sensor',sensor, '   ',Sensor_ID(sensor)
  print*,' total_profile read ',Total_profiles
  print*,' (iproc-1)*Max_Profiles+N_Profiles= ', &
        (iproc-1)*Max_profiles+N_profiles
  print*, 'total profiles written ',iprof_write
  close(lurad(sensor))
  close(luprof(sensor))

  enddo !  sensor

  ! ============================================================================
  ! 7. **** DESTROY THE CRTM ****
  !
  WRITE( *, '( /5x, "Destroying the CRTM..." )' )
  Error_Status = CRTM_Destroy( ChannelInfo )
  IF ( Error_Status /= SUCCESS ) THEN 
    CALL Display_Message( PROGRAM_NAME, &
                          'Error destroying CRTM', & 
                          Error_Status )
  END IF
  ! ============================================================================


  ! ============================================================================
  ! 8. **** CLEAN UP ****
  !
  ! 8a. Deallocate the structures.
  !     These are the explicitly allocated structures.
  !     Note that in some cases other structures, such as the Sfc
  !     and RTSolution structures, will also be allocated and thus
  !     should also be deallocated here.
  ! -------------------------------------------------------------
  Error_Status = CRTM_Destroy_Atmosphere(Atm)

  ! 8b. Deallocate the arrays
  ! -------------------------
  DEALLOCATE(RTSolution, STAT=Allocate_Status)
  ! ============================================================================
  close(luwk1)
  close(luwk2)

END PROGRAM Sim_FORWARD

